// SPDX-License-Identifier: AGPL-3.0-or-later
// SPDX-FileCopyrightText: 2023 swt23w23
package catering.catalog;

import org.salespointframework.catalog.Catalog;
import org.salespointframework.catalog.Product;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.util.Streamable;
import org.springframework.lang.NonNull;

import catering.order.OrderType;
import org.springframework.stereotype.Repository;

@Repository
public interface CateringCatalog extends Catalog<Product> {
	static final Sort DEFAULT_SORT = Sort.sort(Product.class).by(Product::getName).ascending();

	/**
	 * Returns all {@link Product}s by type ordered by the given {@link Sort}.
	 *
	 * @param category an element of {@link OrderType}, must not be {@literal null}.
	 * @param sort     must not be {@literal null}.
	 * @return the consumables with the given category, never {@literal null}.
	 */
	Streamable<Product> findByCategories(OrderType category, Sort sort);

	/**
	 * Returns all {@link Product}s by type ordered by their identifier.
	 *
	 * @param category must not be {@literal null}.
	 * @return the Product with the given category, never {@literal null}.
	 */
	default Streamable<Product> findByCategories(@NonNull OrderType category) {
		return findByCategories(category, DEFAULT_SORT);
	}

	@Query("select p from #{#entityName} p")
	Streamable<Rentable> findRentables();

	@Query("select p from #{#entityName} p")
	Streamable<Consumable> findConsumables();

	Streamable<Rentable> findRentablesByCategoriesContains(@NonNull String category);

	Streamable<Consumable> findConsumablesByCategoriesContains(@NonNull String category);

}
