// SPDX-License-Identifier: AGPL-3.0-or-later
// SPDX-FileCopyrightText: 2023-2024 swt23w23
package catering.catalog;

import java.util.Objects;
import java.util.Set;

import javax.money.MonetaryAmount;

import org.salespointframework.catalog.Product;
import org.salespointframework.quantity.Metric;
import org.springframework.lang.NonNull;
import org.springframework.util.Assert;

import catering.order.OrderType;
import jakarta.persistence.Entity;

@Entity
public class Rentable extends Product {

	private MonetaryAmount wholesalePrice;
	private MonetaryAmount pricePerHour;

	@SuppressWarnings({ "deprecation" })
	public Rentable() {
	}

	public Rentable(String name, MonetaryAmount pricePerHour, MonetaryAmount wholesalePrice,
			Set<OrderType> categories, Metric metric) {
		super(name, pricePerHour, metric);
		this.wholesalePrice = wholesalePrice;
		Assert.notNull(pricePerHour, "pricePerHour must not be null!");
		Assert.notNull(wholesalePrice, "wholesalePrice must not be null!");
		categories.stream().map(OrderType::toString).forEach(this::addCategory);
	}

	public @NonNull MonetaryAmount getWholesalePrice() {
		return wholesalePrice;
	}

	public void setWholesalePrice(MonetaryAmount price) {
		Assert.notNull(price, "wholesalePrice must not be null!");
		wholesalePrice = price;
	}

	public MonetaryAmount getRetailPrice() {
		return super.getPrice();
	}

	public MonetaryAmount getPriceForTime(int hours) {
		return getPrice().multiply(hours).add(getPrice());
	}

	@Override
	public int hashCode() {
		return Objects.hash(Rentable.class, getId());
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof Rentable other) {
			return getId().equals(other.getId());
		}
		return false;
	}
}
