// SPDX-License-Identifier: AGPL-3.0-or-later
// SPDX-FileCopyrightText: 2023-2024 swt23w23
package catering.orderCatalog;

import catering.order.OrderType;
import org.salespointframework.catalog.Product;
import org.salespointframework.inventory.UniqueInventory;
import org.salespointframework.inventory.UniqueInventoryItem;
import org.salespointframework.quantity.Quantity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.*;
import java.util.stream.Collectors;

@Controller
public class CatalogController {
	private final CustomCatalogEntryRepository catalogEntryRepository;
	private Map<Product, Quantity> productMap;
	private CustomCatalogEntry formCatalogEntry;
	private final UniqueInventory<UniqueInventoryItem> inventory;

	public CatalogController(CustomCatalogEntryRepository catalogEntryRepository,
			UniqueInventory<UniqueInventoryItem> inventory) {
		productMap = new HashMap<>();
		formCatalogEntry = new CustomCatalogEntry(
				OrderType.EVENT_CATERING,
				new HashMap<String, Quantity>(),
				0,
				0);
		this.catalogEntryRepository = catalogEntryRepository;
		this.inventory = inventory;
	}

	@GetMapping("/catalog")
	public String catalog(Model model) {
		model.addAttribute("catalogEntries", catalogEntryRepository.findAll());
		return "catalog";
	}

	@GetMapping("/catalog_editor")
	public String editCatalog(Model model) {;
		model.addAttribute("productMap", productMap);
		model.addAttribute("formCatalogEntry", formCatalogEntry);
		model.addAttribute("inventory", inventory.findAll().stream().collect(Collectors.toList()));
		return "catalog_editor";
	}

	@PostMapping("/catalog_editor/addCatalogEntry")
	public String addCatalogEntry(@RequestParam OrderType eventType,
								@RequestParam int minimumTimePeriod,
								@RequestParam int totalCost,
								Model model) {
		Map<String, Quantity> products = new HashMap<String, Quantity>();

		for (Product product : productMap.keySet()) {
			products.put(product.getName(), productMap.get(product));
		}

		catalogEntryRepository.save(new CustomCatalogEntry(
				eventType,
				products,
				minimumTimePeriod,
				totalCost));
		this.formCatalogEntry = new CustomCatalogEntry(
				OrderType.EVENT_CATERING,
				new HashMap<String, Quantity>(),
				0,
				0);
		productMap.clear();
		model.addAttribute("productMap", productMap);
		model.addAttribute("formCatalogEntry", formCatalogEntry);
		return "redirect:/catalog";
	}

	// Removes a catalog entry
	@PostMapping("/catalog/remove")
	public String removeEntry(@RequestParam long catalogEntryID) {
		Optional<CustomCatalogEntry> entry = catalogEntryRepository.findById(catalogEntryID);
		catalogEntryRepository.delete(catalogEntryRepository.findById(catalogEntryID).get());
		return "redirect:/catalog";
	}

	@PostMapping("/catalog_editor/addProduct")
	public String addProduct(@RequestParam("pid") Product product, @RequestParam("number") int number) {
		Quantity amount = Quantity.of(number);
		productMap.compute(product, (key, existingQuantity) ->
			(existingQuantity == null) ? amount : existingQuantity.add(amount));
		formCatalogEntry.addProduct(product.getName(), amount);
		return "redirect:/catalog_editor";
	}

	// Removes a product from the catalog_editor
	@PostMapping("/catalog_editor/removeProduct")
	public String removeProduct(@RequestParam String id, Model model) {

		Iterator<Map.Entry<Product, Quantity>> iterator = productMap.entrySet().iterator();
		while (iterator.hasNext()) {
			Map.Entry<Product, Quantity> currentProduct = iterator.next();
			if (currentProduct.getKey().getId().toString().equals(id)) {
				iterator.remove();
			}
		}
		return "redirect:/catalog_editor";
	}

	@PostMapping("/catalog_editor/addTime")
	public String addTime(@RequestParam int minimumTimePeriod,
						@RequestParam OrderType eventType,
						@RequestParam Map<String, Integer> products,
						Model model) {
		formCatalogEntry.setMinimumTimePeriod(minimumTimePeriod);
		model.addAttribute("formCatalogEntry", formCatalogEntry);
		return "redirect:/catalog_editor";
	}

	@PostMapping("/catalog_editor/chooseEvent")
	public String chooseEvent(String events) {
		switch (events) {
			case "mobile_breakfast":
				formCatalogEntry.setEventType(OrderType.MOBILE_BREAKFAST);
				break;
			case "rent_a_cook":
				formCatalogEntry.setEventType(OrderType.RENT_A_COOK);
				break;
			default:
				formCatalogEntry.setEventType(OrderType.EVENT_CATERING);
				break;
		}

		return "redirect:/catalog_editor";
	}

}
