// SPDX-License-Identifier: AGPL-3.0-or-later
// SPDX-FileCopyrightText: 2023-2024 swt23w23
package catering.orderCatalog;

import catering.order.OrderType;
import jakarta.persistence.*;
import org.salespointframework.quantity.Quantity;

import java.util.Iterator;
import java.util.Map;
import java.util.Objects;

@Entity
public class CustomCatalogEntry {
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "id")
	private Long id;
	private OrderType eventType;

	@ElementCollection
	private Map<String, Quantity> products;
	private int minimumTimePeriod;  // Declared as int for simplification of the prototype. Only whole hours.
	private int totalCost;  // Should actually accumulate the price of all items.

	public CustomCatalogEntry(OrderType eventType, Map<String, Quantity> products, int minimumTimePeriod, int totalCost) {
		this.eventType = eventType;
		this.products = products;
		this.minimumTimePeriod = minimumTimePeriod;
		this.totalCost = totalCost;
	}

	public CustomCatalogEntry() {

	}

	public long getId() {
		return id;
	}

	public OrderType getEventType() {
		return eventType;
	}

	public Map<String, Quantity> getProducts() {
		return products;
	}

	public int getMinimumTimePeriod() {
		return minimumTimePeriod;
	}

	public int getTotalCost() {
		totalCost = 0;
		Iterator<Quantity> iterator = products.values().iterator();
		while (iterator.hasNext()) {
			Quantity currentQuantity = iterator.next();
			totalCost += currentQuantity.getAmount().intValue();
		}
		return totalCost;
	}

	public void setEventType(OrderType eventType) {
		this.eventType = eventType;
	}

	public void addProduct(String name, Quantity count) {
		this.products.put(name, count);
	}

	public void setProducts(Map<String, Quantity> products) {
		this.products = products;
	}
	public void setMinimumTimePeriod(int timePeriod) {
		this.minimumTimePeriod = timePeriod;
	}

	public void setTotalCost(int totalCost) {
		this.totalCost = totalCost;
	}

	@Override
	public int hashCode() {
		return Objects.hash(CustomCatalogEntry.class, getId());
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof CustomCatalogEntry other) {
			return getId() == other.getId();
		}
		return false;
	}
}
