// SPDX-License-Identifier: AGPL-3.0-or-later
// SPDX-FileCopyrightText: 2023 swt23w23
package catering.orderCatalog;

import catering.order.OrderType;
import org.salespointframework.catalog.Product;
import org.salespointframework.core.DataInitializer;
import org.salespointframework.quantity.Quantity;
import org.springframework.stereotype.Component;

import javax.money.CurrencyUnit;
import javax.money.Monetary;
import java.util.HashMap;
import java.util.Map;

@Component
public class CustomCatalogEntryDataInitializer implements DataInitializer {
	private CustomCatalogEntryRepository catalogEntryRepository;

	public CustomCatalogEntryDataInitializer(CustomCatalogEntryRepository catalogEntryRepository) {
		this.catalogEntryRepository = catalogEntryRepository;
	}

	@Override
	public void initialize() {

		Map<Product, Quantity> products = new HashMap<>();
		CurrencyUnit currencyUnit = Monetary.getCurrency("EUR");
		products.put(new Product("Brötchen", Monetary.getDefaultAmountFactory()
			.setCurrency(currencyUnit)
			.setNumber(1)
			.create()), Quantity.of(30));
		products.put(new Product("Kerze", Monetary.getDefaultAmountFactory()
			.setCurrency(currencyUnit)
			.setNumber(2)
			.create()), Quantity.of(20));

		Map<String, Quantity> productsFormatted = new HashMap<String, Quantity>();
		for (Product product : products.keySet()) {
			productsFormatted.put(product.getName(), products.get(product));
		}
		catalogEntryRepository.save(
			new CustomCatalogEntry(
				OrderType.EVENT_CATERING,
				productsFormatted,
				4,
				500));
	}
}
