// SPDX-License-Identifier: AGPL-3.0-or-later
// SPDX-FileCopyrightText: 2023 swt23w23
package catering.users;

import org.salespointframework.core.DataInitializer;
import org.salespointframework.useraccount.UserAccountManagement;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.SessionAttributes;

@Component
@SessionAttributes("profile")
class UserDataInitializer implements DataInitializer {

	private static final Logger LOG = LoggerFactory.getLogger(UserDataInitializer.class);
	private final UserManagement userManagement;
	private final UserAccountManagement userAccountManagement;

	UserDataInitializer(UserManagement userManagement, UserAccountManagement userAccountManagement) {
		Assert.notNull(userManagement, "UserRepository must not be null!");
		this.userManagement = userManagement;
		this.userAccountManagement = userAccountManagement;
	}

	@Override
	public void initialize() {
		if (userAccountManagement.findByUsername("admin").isEmpty()) {
			LOG.info("Creating default user.");
			userManagement.createAdmin("admin", "admin", "admin");
		}
	}
}
