// SPDX-License-Identifier: AGPL-3.0-or-later
// SPDX-FileCopyrightText: 2023-2024 swt23w23
package catering.users;

import java.util.Optional;

import org.salespointframework.useraccount.Password.UnencryptedPassword;
import org.salespointframework.useraccount.Role;
import org.salespointframework.useraccount.UserAccount;
import org.salespointframework.useraccount.UserAccountManagement;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.SessionAttributes;

@Service
@Transactional
@SessionAttributes("profile")
public class UserManagement {

	private final UserRepository users;
	private final UserAccountManagement userAccounts;

	UserManagement(UserRepository users, UserAccountManagement userAccounts) {
		this.users = users;
		this.userAccounts = userAccounts;
	}

	public User createCustomer(
		String username,
		String address,
		String password,
		String fullName
	) {
		return users.save(new User(userAccounts.create(
						username,UnencryptedPassword.of(password),Role.of("CUSTOMER")),address,fullName));
	}

	public User createAdmin(String name, String address, String password) {
		return users.save(new User(userAccounts.create(
						name,UnencryptedPassword.of(password),Role.of("ADMIN")), address, name));
	}

	public User save(User user) {
		return users.save(user);
	}

	public boolean setPassword(String password, UserAccount userAccount) {
		userAccounts.changePassword(userAccount, UnencryptedPassword.of(password));
		return true;
	};

	public UserRepository getUsers() {
		return users;
	}

	public void disableUserAccount(UserAccount userAccount) {
		userAccounts.disable(userAccount.getId());
	}

	public Optional<User> getUserByAccount(UserAccount userAccount) {
		return users.findAll().stream().filter(u -> u.getUserAccount().equals(userAccount)).findFirst();
	}

	public Optional<User> getUserByName(String username) {
		return userAccounts.findByUsername(username).flatMap(ua -> getUserByAccount(ua));
	}
}
